.comment ~
*******************************************************************************
*                                                                             *
*     HarteTec S-100 Super-I/O Controller                                     *
*     Macro Definitions for CP/M3 BIOS Data Structures.                       *
*                                                                             *
*******************************************************************************
*                                                                             *
*  Copyright (C) 2002-2003 Harte Technologies, LLC.                           *
*                                                                             *
*  This file is part of bios3-m80, the CP/M 3.0 CBIOS for the                 *
*  HarteTec S-100 Super-I/O Controller                                        *
*                                                                             *
*  bios3-m80 is free software; you can redistribute it and/or modify          *
*  it under the terms of the GNU General Public License as published by       *
*  the Free Software Foundation; either version 2 of the License, or          *
*  (at your option) any later version.                                        *
*                                                                             *
*  bios3-m80 is distributed in the hope that it will be useful,               *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
*  GNU General Public License for more details.                               *
*                                                                             *
*  You should have received a copy of the GNU General Public License          *
*  along with bios3-m80; if not, write to the Free Software                   *
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
*                                                                             *
*******************************************************************************

 *******************************************************************************
 *                                                                             *
 * $Archive::  $                                                               * 
 *                                                                             *
 * $Workfile:: cpm3.inc                                        $               * 
 *                                                                             *
 * $Revision:: 2                                                      $        *
 *                                                                             *
 * $Author:: Hharte $ http://www.hartetec.com                                  *
 *                                                                             *
 * $Date:: 2/10/04 3:34p                                 $                     *
 *                                                                             *
 *                                                                             *
 * Module Description:                                                         *
 *                                                                             *
 * Environment:                                                                *
 *     kernel mode only                                                        *
 *                                                                             *
 * $History:: cpm3.inc                                         $               *
 * 
 * *****************  Version 2  *****************
 * User: Hharte       Date: 2/10/04    Time: 3:34p
 * Updated in $/SuperIO/bios3-m80
 * Converted Tabs to Spaces for ease of editing under CP/M.
 * 
 * *****************  Version 1  *****************
 * User: Hharte       Date: 8/31/03    Time: 7:49p
 * Created in $/SuperIO/bios3-m80
 * Initial Version
 * 
 *                                                                             *
 *******************************************************************************

~   ; end of .comment

    ; dtbl  <dph0,dph1,...>     - drive table

    ; dph   translate$table,    - disk parameter header
    ;   disk$parameter$block,
    ;   checksum$size,          (optional)
    ;   alloc$size          (optional)

    ; skew  sectors,        - skew table
    ;   skew$factor,
    ;   first$sector$number

    ; dpb   physical$sector$size,   - disk parameter block
    ;   physical$sectors$per$track,
    ;   number$tracks,
    ;   block$size,
    ;   number$dir$entries,
    ;   track$offset,
    ;   checksum$vec$size       (optional)


;   Drive Table.  Contains 16 one word entries.

dtbl macro ?list
    local ?n
?n  defl 0
    irp ?drv,<?list>
?n  defl ?n+1
    defw    ?drv
    endm

    if ?n GT 16
.' Too many drives.  Max 16 allowed'
    exitm
    endc

    if ?n LT 16
    rept (16-?n)
    defw    0
    endm
    endc
 endm

dph macro ?trans,?dpb,?csize,?asize
    local ?csv,?alv
    defw ?trans     ; translate table address
    defb 0,0,0,0,0,0,0,0,0  ; BDOS Scratch area
    defb 0          ; media flag
    defw ?dpb           ; disk parameter block
    if not nul ?csize
    defw ?csv           ; checksum vector
    else
    defw 0FFFEh     ; checksum vector allocated by
    endc            ; GENCPM
    if not nul ?asize
    defw ?alv           ; allocation vector
    else
    defw 0FFFEh     ; alloc vector allocated by GENCPM
    endc
    defw 0fffeh,0fffeh,0fffeh   ; dirbcb, dtabcb, hash alloc'd
                ; by GENCPM
    defb 0          ; hash bank

    if not nul ?csize
?csv:   defs    ?csize      ; checksum vector
    endc
    if not nul ?asize
?alv:   defs    ?asize      ; allocation vector
    endc

    endm

dpb macro ?psize,?pspt,?trks,?bls,?ndirs,?off,?ncks
    local ?spt,?bsh,?blm,?exm,?dsm,?drm,?al0,?al1,?cks,?psh,?psm
    local ?n
;; physical sector mask and physical sector shift
    ?psh    defl 0
    ?n      defl ?psize/128
    ?psm    defl ?n-1
    rept 8
    ?n  defl ?n/2
        if ?n EQ 0 
            exitm
        endc
    ?psh    defl ?psh + 1
    endm
    ?spt    defl ?pspt*(?psize/128)

    ?bsh    defl 3
    ?n      defl ?bls/1024
    rept 8
    ?n  defl ?n/2
        if ?n EQ 0
            exitm
        endc
    ?bsh    defl ?bsh + 1
    endm
    ?blm    defl ?bls/128-1
    ?size   defl (?trks-?off)*?spt
    ?dsm    defl ?size/(?bls/128)-1

    ?exm    defl ?bls/1024
    if ?dsm GT 255
        if ?bls EQ 1024
.'Error, can''t have this size disk with 1k block size'
        exitm
        endc
    ?exm    defl ?exm/2
    endc
    ?exm    defl ?exm-1
    ?all    defl 0
    ?n      defl (?ndirs*32+?bls-1)/?bls
    rept ?n
    ?all    defl (?all shr 1) or 8000h
    endm
    ?al0    defl high ?all
    ?al1    defl low ?all
    ?drm    defl ?ndirs-1
    if not nul ?ncks
        ?cks    defl ?ncks
    else
    ?cks    defl ?ndirs/4
    endc
    defw    ?spt        ; 128 byte records per track
    defb    ?bsh,?blm   ; block shift and mask
    defb    ?exm        ; extent mask
    defw    ?dsm        ; maximum block number
    defw    ?drm        ; maximum directory entry number
    defb    ?al0,?al1   ; alloc vector for directory
    defw    ?cks        ; checksum size
    defw    ?off        ; offset for system tracks
    defb    ?psh,?psm   ; physical sector size shift
                ; and mask
    endm

;
gcd macro ?m,?n
        ;;  greatest common divisor of m,n
            ;;  produces value gcdn as result
            ;;  (used in sector translate table generation)
    ?gcdm   defl ?m ;;variable for m
    ?gcdn   defl ?n ;;variable for n
    ?gcdr   defl 0  ;;variable for r
    rept 65535
    ?gcdx   defl ?gcdm/?gcdn
    ?gcdr   defl ?gcdm - ?gcdx*?gcdn
        if ?gcdr EQ 0
        exitm
        endc
    ?gcdm   defl ?gcdn
    ?gcdn   defl ?gcdr
    endm
    endm

skew macro ?secs,?skf,?fsc
;;  generate the translate table
    ?nxtsec defl 0  ;;next sector to fill
    ?nxtbas defl 0  ;;moves by one on overflow
    gcd %?secs,?skf
    ;;  ?gcdn EQ gcd(?secs,skew)
    ?neltst defl ?secs/?gcdn
    ;;  neltst is number of elements to generate
    ;;  before we overlap previous elements
    ?nelts  defl ?neltst    ;;counter
    rept ?secs  ;;once for each sector
    defb    ?nxtsec+?fsc
    ?nxtsec defl ?nxtsec+?skf
        if ?nxtsec GE ?secs
        ?nxtsec defl ?nxtsec-?secs
        endc
    ?nelts  defl ?nelts-1
        if ?nelts EQ 0
        ?nxtbas defl ?nxtbas+1
        ?nxtsec defl ?nxtbas
        ?nelts  defl ?neltst
        endc
    endm
    endm
