.comment ~
*******************************************************************************
*                                                                             *
*     HarteTec S-100 Super-I/O Controller Definitions                         *
*                                                                             *
*******************************************************************************
*                                                                             *
*  Copyright (C) 2002-2003 Harte Technologies, LLC.                           *
*                                                                             *
*  This file is part of bios3-m80, the CP/M 3.0 CBIOS for the                 *
*  HarteTec S-100 Super-I/O Controller                                        *
*                                                                             *
*  bios3-m80 is free software; you can redistribute it and/or modify          *
*  it under the terms of the GNU General Public License as published by       *
*  the Free Software Foundation; either version 2 of the License, or          *
*  (at your option) any later version.                                        *
*                                                                             *
*  bios3-m80 is distributed in the hope that it will be useful,               *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
*  GNU General Public License for more details.                               *
*                                                                             *
*  You should have received a copy of the GNU General Public License          *
*  along with bios3-m80; if not, write to the Free Software                   *
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
*                                                                             *
*******************************************************************************

 *******************************************************************************
 *                                                                             *
 * $Archive::  $                                                               * 
 *                                                                             *
 * $Workfile:: superio.inc                                     $               * 
 *                                                                             *
 * $Revision:: 4                                                      $        *
 *                                                                             *
 * $Author:: Hharte $ http://www.hartetec.com                                  *
 *                                                                             *
 * $Date:: 2/10/04 3:34p                                 $                     *
 *                                                                             *
 *                                                                             *
 * Module Description:                                                         *
 *                                                                             *
 * Environment:                                                                *
 *     kernel mode only                                                        *
 *                                                                             *
 * $History:: superio.inc                                      $               *
 * 
 * *****************  Version 4  *****************
 * User: Hharte       Date: 2/10/04    Time: 3:34p
 * Updated in $/SuperIO/bios3-m80
 * Converted Tabs to Spaces for ease of editing under CP/M.
 * 
 * *****************  Version 3  *****************
 * User: Hharte       Date: 2/05/04    Time: 7:44p
 * Updated in $/SuperIO/bios3-m80
 * Supports Multiplexed IDE and Floppy INT instead of polling.
 * This version still only supports 720K Diskettes.
 * 
 * *****************  Version 2  *****************
 * User: Hharte       Date: 2/03/04    Time: 11:29p
 * Updated in $/SuperIO/bios3-m80
 * Reads binary files from CP/M.  Programs arbitrary 8K FLASH
 * blocks.
 * 
 * *****************  Version 1  *****************
 * User: Hharte       Date: 8/31/03    Time: 7:49p
 * Created in $/SuperIO/bios3-m80
 * Initial Version
 * 
 *                                                                             *
 *******************************************************************************

~   ; end of .comment

; SMsC 37C93xAPM Config Register 
SUCFGKE EQU 055H        ;SUPER I/O CONFIGURATION KEY (ENTER CONFIG)
SUCFGKX EQU 0AAH        ;SUPER I/O CONFIGURATION KEY (EXIT CONFIG)

SUCFG   EQU 0F0H        ;SUPER I/O CONFIG PORT (R/O)
SUIDX   EQU SUCFG       ;SUPER I/O INDEX PORT (R/O)
SUDATA  EQU SUIDX+1     ;SUPER I/O DATA PORT (R/W)

; SUPER I/O GLOBAL CONFIGURATION REGISTERS
SUGCCTL EQU 2H      ;SUPER I/O CONFIGURATION CONTROL REGISTER
SUGCIDX EQU 3H      ;SUPER I/O INDEX ADDRESS
SUGCLDN EQU 7H      ;SUPER I/O LOGICAL DEVICE NUMBER 
SUGCOSC EQU 24H     ;SUPER I/O OSCILLATOR REGISTER
SUGCCLK EQU 28H     ;SUPER I/O CLOCK MASTER REGISTER

;SUPER I/O LOGICAL DEVICES
LD$FDC  EQU 0H      ;FLOPPY CONTROLLER
LD$IDE0 EQU 1H      ;PRIMARY IDE CONTROLLER
LD$IDE1 EQU 2H      ;SECONDARY IDE CONTROLLER
LD$PAR  EQU 3H      ;PARALLEL PORT
LD$COM0 EQU 5H      ;SERIAL PORT 0
LD$COM1 EQU 4H      ;SERIAL PORT 1
LD$RTC  EQU 6H      ;REAL-TIME CLOCK
LD$KBD  EQU 7H      ;KEYBOARD CONTROLLER
LD$AUX  EQU 8H      ;AUX I/O
LD$ACC  EQU 9H      ;ACCESS.bus

PRI$BASE EQU    60H
SEC$BASE EQU    62H
ACTIVATE EQU    30H


; Port definitions for the IMSAI Super-I/O Controller

p$sup$bsr   equ     080h        ; FLASH Bank Select Register
p$sup$wsr   equ     081h        ; FLASH Window Size Register
p$sup$bar   equ     082h        ; FLASH Base Address Register
p$sup$mux   equ     083h        ; Super-I/O IDE/Serial MUX Register

p$isa$fdc   equ     090h        ; Diskette controller base address

p$ser0$base equ     0A0h        ; Console Serial Port
p$ser1$base equ     0A8h        ; AUX Serial Port

IDE$BASE    equ     0B0h        ; base address of IDE Controllers

p$ide$DOR   equ     IDE$BASE+6   ; Digital Output Register
p$ide$Dat   equ     IDE$BASE+0   ; Data Register
p$ide$Err   equ     IDE$BASE+1   ; Error Register
p$ide$PCmp  equ     p$ide$Err
p$ide$SCnt  equ     IDE$BASE+02h ; Sector Count
p$ide$SNum  equ     IDE$BASE+03h ; Sector Number
p$ide$CLo   equ     IDE$BASE+04h ; Cylinder Low
p$ide$CHi   equ     IDE$BASE+05h ; Cylinder High
p$ide$SDH   equ     IDE$BASE+06h ; Drive and Head
p$ide$Cmd   equ     IDE$BASE+07h ; Command / Status
p$ide$Stat  equ     p$ide$Cmd

; Definitions for FLASH Window Size

p$wsz$4k    equ 1       ; 4K Window Size
p$wsz$8k    equ 2       ; 8K Window Size
p$wsz$16k   equ 3       ; 16K Window Size
p$wsz$32k   equ 4       ; 32K Window Size
p$wsz$64k   equ 5       ; 64K Window Size
p$wsz$128k  equ 6       ; 128K Window Size

; Diskette controller Registers

p$fdc$sra       equ     p$isa$fdc+0 ; status register A     (R)
p$fdc$srb       equ     p$fdc$sra+1 ; status register B     (R/W)
p$fdc$dor       equ     p$fdc$sra+2 ; digital output reg    (R/W)
p$fdc$tdr       equ     p$fdc$sra+3 ; tape drive register   (R/W)
p$fdc$msr       equ     p$fdc$sra+4 ; Main Status register  (R)
p$fdc$dsr       equ     p$fdc$sra+4 ; Datarate Select Reg   (W)
p$fdc$fifo      equ     p$fdc$sra+5 ; Data Fifo             (R/W)
p$fdc$resvd     equ     p$fdc$sra+6 ; Reserved
p$fdc$dir       equ     p$fdc$sra+7 ; digital input reg     (R)
p$fdc$ccr       equ     p$fdc$sra+7 ; configuration ctl reg (W)


; some bit definitions

; Main Status Register (MSR) bit definitions

b$msr$rqm       equ     080h    ; Set to '1' if can write to FDC
b$msr$dio       equ     040h    ; direction of data to FDC: 1=r, 0=w.
b$msr$nondma    equ     020h    ; for polled data transfers.
b$msr$cb        equ     010h    ; Command Busy. 1=cmd in progress
b$msr$drvbusy   equ     00Fh    ; set to '1' when corresponding drive is seeking

b$dor$reset     equ     004h    ; '0' resets the FDC

b$ccr$dr250k    equ     002h    ; 250Kb/s data rate
b$ccr$dr500k    equ     000h    ; 500Kb/s data rate

b$st0$se        equ     020h    ; SE - seek end bit.

fdc$cmd$specify equ     003h    ; the "Specify" command.
fdc$cmd$config  equ     013h    ; the "Configure" command.
fdc$cmd$format  equ     04Dh    ; the "Format" command.


;_DMA   equ 0a0h    ;Diskette DMA address


; 16550-Compatible UARTS Port definitions
p$ser0$data equ p$ser0$base+0
p$ser0$stat equ p$ser0$base+5
p$ser1$data equ p$ser1$base+0
p$ser1$stat equ p$ser1$base+5

; 16550 UART Registers
p$ser$thr       equ     0               ; transmit holding register (W/O)
p$ser$rhr       equ     0               ; receive holding register  (R/O)
p$ser$lsb       equ     0               ; LSB baud rate divisor     (W/O)
p$ser$ier       equ     1               ; interrupt enable register (W/O)
p$ser$msb       equ     1               ; MSB baud rate divisor     (W/O)
p$ser$fcr       equ     2               ; FIFO control register     (W/O)
p$ser$isr       equ     2               ; interrupt status register (R/O)
p$ser$lcr       equ     3               ; line control register     (W/O)
p$ser$mcr       equ     4               ; modem control register    (W/O)
p$ser$lsr       equ     5               ; line status register      (R/O)
p$ser$msr       equ     6               ; modem status register     (R/O)
p$ser$spr       equ     7               ; scratchpad register       (R/W)

b$ser$baudl     equ     080h            ; baud rate register latch





